@@ -53,15 +53,9 @@ class Agents(ScrapiBase):
             self.agent_id = agent_id
             self.client_options = self._set_region(agent_id)
 
-    def list_agents(self, location_id: str) -> List[types.Agent]:
-        """Get list of all CX agents in a given GCP project
+    def _build_list_agents_client_request(self, location_id):
+        """Builds the List Agents Request object."""
 
-        Args:
-          location_id: The GCP Project/Location ID in the following format
-              `projects/<GCP PROJECT ID>/locations/<LOCATION ID>
-        Returns:
-          agents: List of Agent objects
-        """
         request = types.agent.ListAgentsRequest()
         request.parent = location_id
 
@@ -70,12 +64,66 @@ class Agents(ScrapiBase):
             credentials=self.creds, client_options=client_options
         )
 
-        response = client.list_agents(request)
+        return client, request
 
-        agents = []
-        for page in response.pages:
-            for agent in page.agents:
-                agents.append(agent)
+    def list_agents(
+        self,
+        location_id: str = None,
+        project_id: str = None) -> List[types.Agent]:
+        """Get list of all CX agents in a given GCP Region or Project.
+
+        This method allows you to provide a specific Location ID consisting of
+        a GCP Project ID and Location ID (i.e. GCP Region Name) to retrieve all
+        of the CX agents associated with that Project/Region. Optionally, you
+        can provide just the Project ID and the funciton will traverse ALL
+        available GCP regions to list ALL agents across the regions.
+
+        Args:
+          location_id: The GCP Project/Location ID in the following format
+              `projects/<GCP PROJECT ID>/locations/<LOCATION ID>`
+              `projects/my-gcp-project/locations/us-central1`
+          project_id: The GCP Project ID as a string
+        Returns:
+          agents: List of Agent objects
+        """
+
+        if project_id:
+            region_list = [
+                "global",
+                "us-central1",
+                "us-east1",
+                "us-west1",
+                "asia-northeast1",
+                "asia-south1",
+                "australia-southeast1",
+                "northamerica-northeast1",
+                "europe-west1",
+                "europe-west2",
+            ]
+
+            agents = []
+            for region in region_list:
+                location_path = "projects/{}/locations/{}".format(
+                    project_id, region
+                )
+
+                client, request = self._build_list_agents_client_request(
+                    location_path
+                )
+
+                agents += self.list_agents(location_id=location_path)
+
+        else:
+            client, request = self._build_list_agents_client_request(
+                location_id
+            )
+
+            response = client.list_agents(request)
+
+            agents = []
+            for page in response.pages:
+                for agent in page.agents:
+                    agents.append(agent)
 
         return agents
 
@@ -93,6 +141,77 @@ class Agents(ScrapiBase):
 
         return response
 
+    def get_agent_by_display_name(
+        self,
+        project_id: str,
+        display_name: str,
+        location_id: str = None,
+        region: str = None
+    ) -> types.Agent:
+        """Get CX agent in a given GCP project by its human readable
+            display name.
+
+        Args:
+          project_id: The GCP Project ID as string
+          display_name: human-readable display name of CX agent as string
+          location_id: Optional. The GCP Project/Location ID, as string
+              and in this format:
+              `projects/<GCP PROJECT ID>/locations/<LOCATION ID>`
+              Improves execution time and resolves conflicts caused
+              when multiple agents on different regions have identical
+              display names.
+          region: Optional. The agent's region ID as string.
+              Improves execution time and resolves conflicts caused
+              when multiple agents on different regions have identical
+              display names.
+              Syntax for region ID can be found here:
+              https://cloud.google.com/dialogflow/cx/docs/concept/region#avail
+        Returns:
+          Agents: CX agent resource object. If no agent is found,
+              returns None.
+        """
+
+        if location_id:
+            agent_list = self.list_agents(
+                location_id=location_id
+                )
+
+        elif region:
+            agent_list = self.list_agents(
+                location_id="projects/{}/locations/{}".format(
+                    project_id, region
+                    )
+            )
+        else:
+            agent_list = self.list_agents(project_id=project_id)
+
+        possible_agent = None
+        matched_agent = None
+
+        for agent in agent_list:
+            if agent.display_name == display_name and not matched_agent:
+                matched_agent = agent
+            elif agent.display_name == display_name and matched_agent:
+                possible_agent = agent
+            elif agent.display_name.lower() == display_name.lower():
+                possible_agent = agent
+
+        if possible_agent and not matched_agent:
+            logging.warning(
+                "display_name is case-sensitive. Did you mean \"%s\"?",
+                possible_agent.display_name
+            )
+        elif possible_agent and matched_agent:
+            logging.warning(
+                '''Found multiple agents with the display name \"%s\".
+                 Include location_id or region parameter to resolve cross-region
+                 ambiguity.''',
+                possible_agent.display_name
+            )
+            matched_agent = None
+
+        return matched_agent
+
     def create_agent(
         self,
         project_id: str,
@@ -240,7 +359,8 @@ class Agents(ScrapiBase):
 
         Returns:
           response: A Long Running Operation (LRO) ID that can be used to
-            check the status of the export using dfcx.get_lro()
+            check the status of the export using
+              dfcx_scrapi.core.operations->get_lro()
         """
 
         request = types.agent.ExportAgentRequest()
@@ -270,7 +390,8 @@ class Agents(ScrapiBase):
 
         Returns:
           response: A Long Running Operation (LRO) ID that can be used to
-            check the status of the import using dfcx.get_lro()
+            check the status of the import using
+              dfcx_scrapi.core.operations->get_lro()
         """
 
         request = types.RestoreAgentRequest()
